Attribute VB_Name = "Resize"
Option Explicit

Private Declare Function GetObject Lib "gdi32" Alias "GetObjectA" (ByVal hObject As Long, ByVal nCount As Long, lpObject As Any) As Long
Private Declare Function GetDIBits Lib "gdi32" (ByVal aHDC As Long, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long, lpBits As Any, lpBI As BITMAPINFO, ByVal wUsage As Long) As Long
Private Declare Function SetDIBits Lib "gdi32" (ByVal hdc As Long, ByVal hBitmap As Long, ByVal nStartScan As Long, ByVal nNumScans As Long, lpBits As Any, lpBI As BITMAPINFO, ByVal wUsage As Long) As Long
Private Type BITMAP '14 bytes
    bmType As Long
    bmWidth As Long
    bmHeight As Long
    bmWidthBytes As Long
    bmPlanes As Integer
    bmBitsPixel As Integer
    bmBits As Long
End Type
Private Type BITMAPINFOHEADER '40 bytes
    biSize As Long
    biWidth As Long
    biHeight As Long
    biPlanes As Integer
    biBitCount As Integer
    biCompression As Long
    biSizeImage As Long
    biXPelsPerMeter As Long
    biYPelsPerMeter As Long
    biClrUsed As Long
    biClrImportant As Long
End Type
Private Type RGBQUAD
    rgbBlue As Byte
    rgbGreen As Byte
    rgbRed As Byte
    rgbReserved As Byte
End Type
Private Type BITMAPINFO
    bmiHeader As BITMAPINFOHEADER
    bmiColors As RGBQUAD
End Type
Private Const DIB_RGB_COLORS = 0&
Private Const BI_RGB = 0&

Private Const pixR As Integer = 3
Private Const pixG As Integer = 2
Private Const pixB As Integer = 1

' Shrink or enlarge the picture.
Public Sub ResizePicture(ByVal pic_from As PictureBox, ByVal pic_to As PictureBox, ByVal from_xmin As Single, ByVal from_ymin As Single, ByVal from_wid As Single, ByVal from_hgt As Single, ByVal to_xmin As Single, ByVal to_ymin As Single, ByVal to_wid As Single, ByVal to_hgt As Single)
Dim x_scale As Single
Dim y_scale As Single

    ' If either scale is less than 1, use ShrinkPicture
    If (to_wid / from_wid < 1#) Or _
       (to_hgt / from_hgt < 1#) _
    Then
        ' Shrink the picture.
        ShrinkPicture pic_from, pic_to, _
            from_xmin, from_ymin, _
            from_wid, from_hgt, _
            to_xmin, to_ymin, _
            to_wid, to_hgt
    Else
        ' Enlarge the picture.
        EnlargePicture pic_from, pic_to, _
            from_xmin, from_ymin, _
            from_wid, from_hgt, _
            to_xmin, to_ymin, _
            to_wid, to_hgt
    End If
End Sub
' Shrink the image.
Private Sub ShrinkPicture(ByVal pic_from As PictureBox, ByVal pic_to As PictureBox, ByVal from_xmin As Single, ByVal from_ymin As Single, ByVal from_wid As Single, ByVal from_hgt As Single, ByVal to_xmin As Single, ByVal to_ymin As Single, ByVal to_wid As Single, ByVal to_hgt As Single)
Dim in_bitmap_info As BITMAPINFO
Dim in_pixels() As Byte
Dim in_bytes_per_scanLine As Integer
Dim in_pad_per_scanLine As Integer
Dim out_bitmap_info As BITMAPINFO
Dim out_pixels() As Byte
Dim out_bytes_per_scanLine As Integer
Dim out_pad_per_scanLine As Integer
Dim x_scale As Single
Dim y_scale As Single
Dim ix_max As Single
Dim iy_max As Single
Dim x_in As Single
Dim y_in As Single
Dim ix_out As Integer
Dim iy_out As Integer
Dim ix_in As Integer
Dim iy_in As Integer
Dim x1 As Single
Dim x2 As Single
Dim y1 As Single
Dim y2 As Single
Dim X As Integer
Dim Y As Integer
Dim r As Integer
Dim g As Integer
Dim b As Integer
Dim num_pixels As Integer

    ' Get the input image's pixels.
    With in_bitmap_info.bmiHeader
        .biSize = 40
        .biWidth = pic_from.ScaleWidth
        ' Use negative height to scan top-down.
        .biHeight = -pic_from.ScaleHeight
        .biPlanes = 1
        .biBitCount = 32
        .biCompression = BI_RGB
        in_bytes_per_scanLine = ((((.biWidth * .biBitCount) + 31) \ 32) * 4)
        in_pad_per_scanLine = in_bytes_per_scanLine - (((.biWidth * .biBitCount) + 7) \ 8)
        .biSizeImage = in_bytes_per_scanLine * Abs(.biHeight)
    End With
    ReDim in_pixels(1 To 4, 0 To pic_from.ScaleWidth - 1, 0 To pic_from.ScaleHeight - 1)
    GetDIBits pic_from.hdc, pic_from.Image, _
        0, pic_from.ScaleHeight, in_pixels(1, 0, 0), _
        in_bitmap_info, DIB_RGB_COLORS

    ' Prepare to save the output image's pixels.
    With out_bitmap_info.bmiHeader
        .biSize = 40
        .biWidth = pic_to.ScaleWidth
        ' Use negative height to scan top-down.
        .biHeight = -pic_to.ScaleHeight
        .biPlanes = 1
        .biBitCount = 32
        .biCompression = BI_RGB
        out_bytes_per_scanLine = ((((.biWidth * .biBitCount) + 31) \ 32) * 4)
        out_pad_per_scanLine = out_bytes_per_scanLine - (((.biWidth * .biBitCount) + 7) \ 8)
        .biSizeImage = out_bytes_per_scanLine * Abs(.biHeight)
    End With
    ReDim out_pixels(1 To 4, 0 To pic_to.ScaleWidth - 1, 0 To pic_to.ScaleHeight - 1)

    ' Get the original image's bounds.
    ix_max = pic_from.ScaleWidth - 2
    iy_max = pic_from.ScaleHeight - 2

    ' Calulate the mapping values.
    from_xmin = from_xmin
    from_ymin = from_ymin
    to_xmin = to_xmin
    to_ymin = to_ymin
    x_scale = to_wid / (from_wid - 1)
    y_scale = to_hgt / (from_hgt - 1)

    ' Calculate the output pixel values.
    For iy_out = 0 To pic_to.ScaleHeight - 1
        For ix_out = 0 To pic_to.ScaleWidth - 1
            ' Map the pixel value from
            ' (ix_out, iy_out) to (x_in, y_in).
            x1 = Int(from_xmin + (ix_out - to_xmin) / x_scale)
            x2 = Int(from_xmin + (ix_out + 1 - to_xmin) / x_scale) - 1
            y1 = Int(from_ymin + (iy_out - to_ymin) / y_scale)
            y2 = Int(from_ymin + (iy_out + 1 - to_ymin) / y_scale) - 1

            ' Average the pixels in this area.
            r = 0
            g = 0
            b = 0
            For X = x1 To x2
                For Y = y1 To y2
                    r = r + in_pixels(pixR, X, Y)
                    g = g + in_pixels(pixG, X, Y)
                    b = b + in_pixels(pixB, X, Y)
                Next Y
            Next X

            ' Save the result.
            num_pixels = (x2 - x1 + 1) * (y2 - y1 + 1)
            out_pixels(pixR, ix_out, iy_out) = r / num_pixels
            out_pixels(pixG, ix_out, iy_out) = g / num_pixels
            out_pixels(pixB, ix_out, iy_out) = b / num_pixels
        Next ix_out
    Next iy_out

    ' Set pic_to's pixels.
    SetDIBits pic_to.hdc, pic_to.Image, _
        0, pic_to.ScaleHeight, out_pixels(1, 0, 0), _
        out_bitmap_info, DIB_RGB_COLORS
    pic_to.Picture = pic_to.Image
End Sub
' Enlarge the image.
Private Sub EnlargePicture(ByVal pic_from As PictureBox, ByVal pic_to As PictureBox, ByVal from_xmin As Single, ByVal from_ymin As Single, ByVal from_wid As Single, ByVal from_hgt As Single, ByVal to_xmin As Single, ByVal to_ymin As Single, ByVal to_wid As Single, ByVal to_hgt As Single)
Dim in_bitmap_info As BITMAPINFO
Dim in_pixels() As Byte
Dim in_bytes_per_scanLine As Integer
Dim in_pad_per_scanLine As Integer
Dim out_bitmap_info As BITMAPINFO
Dim out_pixels() As Byte
Dim out_bytes_per_scanLine As Integer
Dim out_pad_per_scanLine As Integer
Dim x_scale As Single
Dim y_scale As Single
Dim ix_max As Single
Dim iy_max As Single
Dim x_in As Single
Dim y_in As Single
Dim ix_out As Integer
Dim iy_out As Integer
Dim ix_in As Integer
Dim iy_in As Integer
Dim dx As Single
Dim dy As Single
Dim dx1 As Single
Dim dx2 As Single
Dim dy1 As Single
Dim dy2 As Single
Dim v11 As Integer
Dim v12 As Integer
Dim v21 As Integer
Dim v22 As Integer

    ' Get the input image's pixels.
    With in_bitmap_info.bmiHeader
        .biSize = 40
        .biWidth = pic_from.ScaleWidth
        ' Use negative height to scan top-down.
        .biHeight = -pic_from.ScaleHeight
        .biPlanes = 1
        .biBitCount = 32
        .biCompression = BI_RGB
        in_bytes_per_scanLine = ((((.biWidth * .biBitCount) + 31) \ 32) * 4)
        in_pad_per_scanLine = in_bytes_per_scanLine - (((.biWidth * .biBitCount) + 7) \ 8)
        .biSizeImage = in_bytes_per_scanLine * Abs(.biHeight)
    End With
    ReDim in_pixels(1 To 4, 0 To pic_from.ScaleWidth - 1, 0 To pic_from.ScaleHeight - 1)
    GetDIBits pic_from.hdc, pic_from.Image, _
        0, pic_from.ScaleHeight, in_pixels(1, 0, 0), _
        in_bitmap_info, DIB_RGB_COLORS

    ' Prepare to save the output image's pixels.
    With out_bitmap_info.bmiHeader
        .biSize = 40
        .biWidth = pic_to.ScaleWidth
        ' Use negative height to scan top-down.
        .biHeight = -pic_to.ScaleHeight
        .biPlanes = 1
        .biBitCount = 32
        .biCompression = BI_RGB
        out_bytes_per_scanLine = ((((.biWidth * .biBitCount) + 31) \ 32) * 4)
        out_pad_per_scanLine = out_bytes_per_scanLine - (((.biWidth * .biBitCount) + 7) \ 8)
        .biSizeImage = out_bytes_per_scanLine * Abs(.biHeight)
    End With
    ReDim out_pixels(1 To 4, 0 To pic_to.ScaleWidth - 1, 0 To pic_to.ScaleHeight - 1)

    ' Get the original image's bounds.
    ix_max = pic_from.ScaleWidth - 2
    iy_max = pic_from.ScaleHeight - 2

    ' Calulate the mapping values.
    from_xmin = from_xmin
    from_ymin = from_ymin
    to_xmin = to_xmin
    to_ymin = to_ymin
    x_scale = to_wid / (from_wid - 1)
    y_scale = to_hgt / (from_hgt - 1)

    ' Calculate the output pixel values.
    For iy_out = 0 To pic_to.ScaleHeight - 1
        For ix_out = 0 To pic_to.ScaleWidth - 1
            ' Map the pixel value from
            ' (ix_out, iy_out) to (x_in, y_in).
            x_in = from_xmin + (ix_out - to_xmin) / x_scale
            y_in = from_ymin + (iy_out - to_ymin) / y_scale

            ' Interpolate to find the pixel's value.
            ' Find the nearest integral position.
            ix_in = Int(x_in)
            iy_in = Int(y_in)

            ' See if this is out of bounds.
            If (ix_in < 0) Or (ix_in > ix_max) Or _
               (iy_in < 0) Or (iy_in > iy_max) _
            Then
                ' The point is outside the image.
                ' Use white.
                out_pixels(pixR, ix_out, iy_out) = 255
                out_pixels(pixG, ix_out, iy_out) = 255
                out_pixels(pixB, ix_out, iy_out) = 255
            Else
                ' The point lies within the image.
                ' Calculate its value.
                dx1 = x_in - ix_in
                dy1 = y_in - iy_in
                dx2 = 1# - dx1
                dy2 = 1# - dy1

                ' Calculate the red value.
                v11 = in_pixels(pixR, ix_in, iy_in)
                v12 = in_pixels(pixR, ix_in, iy_in + 1)
                v21 = in_pixels(pixR, ix_in + 1, iy_in)
                v22 = in_pixels(pixR, ix_in + 1, iy_in + 1)
                out_pixels(pixR, ix_out, iy_out) = _
                    v11 * dx2 * dy2 + v12 * dx2 * dy1 + _
                    v21 * dx1 * dy2 + v22 * dx1 * dy1
    
                ' Calculate the green value.
                v11 = in_pixels(pixG, ix_in, iy_in)
                v12 = in_pixels(pixG, ix_in, iy_in + 1)
                v21 = in_pixels(pixG, ix_in + 1, iy_in)
                v22 = in_pixels(pixG, ix_in + 1, iy_in + 1)
                out_pixels(pixG, ix_out, iy_out) = _
                    v11 * dx2 * dy2 + v12 * dx2 * dy1 + _
                    v21 * dx1 * dy2 + v22 * dx1 * dy1

                ' Calculate the blue value.
                v11 = in_pixels(pixB, ix_in, iy_in)
                v12 = in_pixels(pixB, ix_in, iy_in + 1)
                v21 = in_pixels(pixB, ix_in + 1, iy_in)
                v22 = in_pixels(pixB, ix_in + 1, iy_in + 1)
                out_pixels(pixB, ix_out, iy_out) = _
                    v11 * dx2 * dy2 + v12 * dx2 * dy1 + _
                    v21 * dx1 * dy2 + v22 * dx1 * dy1
            End If
        Next ix_out
    Next iy_out

    ' Set pic_to's pixels.
    SetDIBits pic_to.hdc, pic_to.Image, _
        0, pic_to.ScaleHeight, out_pixels(1, 0, 0), _
        out_bitmap_info, DIB_RGB_COLORS
    pic_to.Picture = pic_to.Image
End Sub
